package de.korap.services.utils;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.PrintStream;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.util.Locale;

import java.util.Properties;
import java.util.Set;
import java.util.Map.Entry;

import org.apache.commons.io.FileUtils;


public final class Utils
{
	
	/*
	 * set Err and Out Streams.
	 * 
	 * 12.12.19/FB
	 * The PrintWriter class should be used in situations that require writing characters rather than bytes.
	 * 05.10.21/FB
	 */
	
	public static void setStdFileStreams(String fnameErr, String fnameOut)
	
	{
	FileOutputStream
		fosOut, fosErr;

	try { // open in append mode:
		fosErr = new FileOutputStream(fnameErr, true);
		System.setErr(new PrintStream(fosErr));
		fosOut = new FileOutputStream(fnameOut, true);
		System.setOut(new PrintStream(fosOut));
		} 
	catch (FileNotFoundException e) {
		e.printStackTrace();
		}
	
	} // setStdFileStreams
	
	/*
	 * setLogStream:
	 * 
	 * - open a PrintStream for fname.
	 * 
	 * 25.10.21/FB
	 */
	
	public static PrintStream setLogStream(String fname)
	
	{
	PrintStream
		f = null;
	
	try {
		f = new PrintStream(new FileOutputStream(fname, true));
		} 
	catch (FileNotFoundException e) {
		e.printStackTrace();
		f = System.out;
		f.printf("Error: setLogStream: cannot create PrintStream for '%s'!\n", fname);
		}
	
	return f;
	} // setLogStream

	/*
	 * executeShellScript.
	 * 12.12.19/FB
	 */
	public static int executeShellScript(String fnameScript, String arg, final PrintStream fout, final PrintStream ferr)
	
	{
	final String 
		func = "executeShellScript";
	ProcessBuilder 	
		builder;
    Process 
    	process = null;
    int 
    	status;
    
    fout.printf("Debug: %s: script name = '%s'...\n", func, fnameScript);
	fout.printf("Debug: %s: script arg  = '%s'...\n", func, arg);
		
	if (!new File(fnameScript).canExecute()) {
        System.err.printf("GlemmUtils.executeShellScript: cannot find shell script '%s'!\n", fnameScript);
        return 1;
	}
	
	builder = new ProcessBuilder(fnameScript, arg);
    process = null;
    
    try {
        process = builder.start();
        status = process.waitFor();
        
        fout.println("Debug: script executed successfully.\n");

        //InputStream is = process.getInputStream();
        //InputStreamReader isr = new InputStreamReader(is);
        //BufferedReader br = new BufferedReader(isr);
        //String line;
        //while ((line = br.readLine()) != null) {
        //    System.out.println(line);
        //}
    	}
    catch (IOException e) {
    	ferr.printf("Error: executeShellScript: '%s' failed due to IOException!\n", fnameScript); 
    	e.printStackTrace();
    	return 1;
    }
    catch (InterruptedException e) {
    	ferr.printf("Error: executeShellScript: '%s' failed due to Interrupt!\n", fnameScript); 
    	e.printStackTrace();
    	return 1;
    }
        
	// OK
	return 0;
		
	}

	/*
	 * readFromInputStream:
	 * 
	 * - reads full content of a file into a string and return that string.
	 * - only apply on small files.
	 * 
	 * 27.04.20/FB
	 */
	
	public static String readFromInputStream(String filename)
			  throws IOException 
	
	{
    byte[]
    	encoded;
    
    encoded = Files.readAllBytes(Paths.get(filename));
    return new String(encoded);
    
	/* alternative:  
    try (BufferedReader br = new BufferedReader(new InputStreamReader(ins))) 
    	{
        String line;
        while ((line = br.readLine()) != null) 
        	{
            resultStringBuilder.append(line).append("\n");
        	}
    	}
    return resultStringBuilder.toString();
    */
	
	} // readFromInputStream

	/*
	 * writeToOutputStream:
	 * - writes a String to file.
	 * 27.04.10/FB
	 */
	
	public static void writeToOutputStream(String filename, String s, final PrintStream fout)
	
	{
	final String func = "writeToOutputStream";
	File
		file = new File(filename);
	
	try {
		FileUtils.writeStringToFile(file, s, (String)null, false);
		} 
	catch (IOException e) {
		fout.printf("Error: %s: cannot write to file '%s'!\n", func, filename);
		e.printStackTrace();
		}
	
	} // writeToOutputStream
	
	/*
	 * loadProperties
	 * 
	 * - load properties using InputStream is.
	 * - InputStream must be opened by calling Services using their Servlet Context.
	 * - name used to open InputStream is fnameProps (used here only for logging).
	 * - realPath only available if fnamesProps was found (used here only for logging).
	 * - bPrint: if true: log properties found.
	 * Returns:
	 * - set of loaded properties. May the null if fnameProps were not found.
	 * 15.10.21/FB
	 * Notes:
	 * - as long as the properties are loading, we don't know where to redirect
	 *   System.out/err, so we have to write to System.out/err.
	 * 27.10.21/FB
	 */
	
	public static Properties loadProperties(InputStream is, String fnameProps, String realPath, boolean bPrint)
	
	{
	final String
		func = "Utils.loadProperties";
	Properties
		props = new Properties();
	
	/* does not find the properties file:
	 * inProp = GlemmServices.class.getClassLoader().getResourceAsStream(propName);
	 * inProp = HttpServletRequest.class.getResourceAsStream(propName);
	 */
	
	if( is == null )
		{
		System.err.printf("Error: %s: cannot load '%s' (realPath='%s')!\n", func, fnameProps, realPath);
		return props; // return empty list of props.
		}
	
	try {
		props.load(is);
		Set<Entry<Object,Object>>
			propsSet = props.entrySet();
		
		for(Entry<Object, Object> propEntry : propsSet)
			{
			if( bPrint )
				System.out.printf("Debug: %s: prop %s = '%s'.\n", func, propEntry.getKey(), propEntry.getValue());
			}
		} 
	catch (IOException e) {
		System.err.printf("Error: %s: cannot load props from '%s'!",  func, fnameProps);
		e.printStackTrace();
		}

	return props;	
	} // loadProperties
	
	/*
	 * getConstantFromProperties
	 * 
	 * if key prop is found in props, return val found.
	 * else return default value propDefault.
	 * 
	 * 15.10.21/FB
	 * Notes:
	 * - if loging is reactivated here, do not use System.out/err as they are set JVM-wide.
	 */
	
	public static String getConstantFromProperties(Properties props, String prop, String propDefault)
	
	{
	final String
		func = "init_ConstantFromProperties";
	String
		val = (String)props.getProperty(prop);
	
	if( val != null )
		{
		//System.out.printf("Debug: %s: %s = '%s' (from properties).\n",  func, prop, val);
		return val;
		}	
	else
		{
		//System.err.printf("Error: %s: set constant %s = '%s' (from default)!\n", func, prop, propDefault);
		return propDefault;
		}
	
	} // init_ConstantFromProperties
	

}